@@ -1,4 +1,4 @@
-/* $Id: timer.c 2061 2008-06-20 07:00:39Z aturner $ */
+/* $Id: timer.c 2068 2008-11-09 06:00:06Z aturner $ */
 
 /*
  * Copyright (c) 2001-2007 Aaron Turner.
@@ -66,7 +66,7 @@ void timesdiv(struct timespec *tvs, float div)
         
     interval = ((double)tvs->tv_sec * 1000000000 + tvs->tv_nsec) / (double)div;
     tvs->tv_sec = interval / (int)1000000000;
-    tvs->tv_nsec = interval - (tvs->tv_nsec * 1000000000);
+    tvs->tv_nsec = interval - (tvs->tv_sec * 1000000000);
 }
 
 void
