@@ -1,4 +1,4 @@
-/* $Id: timer.h 2061 2008-06-20 07:00:39Z aturner $ */
+/* $Id: timer.h 2071 2008-11-09 08:36:02Z aturner $ */
 
 /*
  * Copyright (c) 2001-2007 Aaron Turner.
@@ -83,6 +83,14 @@ void timesdiv(struct timespec *tvs, float div);
     } while (0)
 #endif
 
+/* timesec to float */
+#ifndef timer2float
+#define timer2float(tvp, time)                  \
+    do {                                        \
+        time = tvp->tv_sec;                     \
+        time += (float)(tvp->tv_usec / 10000) * 0.01;  \
+    } while (0)
+#endif
 
 #ifndef TIMEVAL_TO_TIMESPEC
 #define TIMEVAL_TO_TIMESPEC(tv, ts) {           \
