@@ -1,4 +1,4 @@
-/* $Id: xX.c 2074 2008-11-09 08:42:24Z aturner $ */
+/* $Id: xX.c 2285 2009-05-03 07:27:38Z aturner $ */
 
 /*
  * Copyright (c) 2001-2004 Aaron Turner.
@@ -41,6 +41,7 @@
 #include "config.h"
 #include "defines.h"
 #include "common.h"
+#include <stdlib.h>
 
 /**
  * returns the include_exclude_mode on success placing the CIDR or LIST in mybuf
@@ -61,18 +62,21 @@ parse_xX_str(tcpr_xX_t *xX, char *str, tcpr_bpf_t *bpf)
         if (!parse_cidr(&(xX->cidr), str, ","))
             return xXError;
         break;
+
     case 'D':                  /* dst ip */
         str = str + 2;
         out = xXDest;
         if (!parse_cidr(&(xX->cidr), str, ","))
             return xXError;
         break;
+
     case 'E':                  /* either ip */
         str = str + 2;
         out = xXEither;
         if (!parse_cidr(&(xX->cidr), str, ","))
             return xXError;
         break;
+        
     case 'F':                  /* bpf filter */
         str = str + 2;
         out = xXBPF;
@@ -83,12 +87,14 @@ parse_xX_str(tcpr_xX_t *xX, char *str, tcpr_bpf_t *bpf)
          * to compile the BPF once we open the pcap file
          */
         break;
+        
     case 'P':                  /* packet id */
         str = str + 2;
         out = xXPacket;
         if (!parse_list(&(xX->list), str))
             return xXError;
         break;
+
     case 'S':                  /* source ip */
         str = str + 2;
         out = xXSource;
@@ -96,7 +102,6 @@ parse_xX_str(tcpr_xX_t *xX, char *str, tcpr_bpf_t *bpf)
             return xXError;
         break;
 
-
     default:
         errx(-1, "Invalid -%c option: %c", xX->mode, *str);
         break;
@@ -120,7 +125,7 @@ parse_xX_str(tcpr_xX_t *xX, char *str, tcpr_bpf_t *bpf)
  * and return 1 if we should send the packet or 0 if not
  */
 int
-process_xX_by_cidr(int mode, tcpr_cidr_t * cidr, ipv4_hdr_t * ip_hdr)
+process_xX_by_cidr_ipv4(int mode, tcpr_cidr_t * cidr, ipv4_hdr_t * ip_hdr)
 {
 
     if (mode & xXExclude) {
@@ -132,12 +137,15 @@ process_xX_by_cidr(int mode, tcpr_cidr_t * cidr, ipv4_hdr_t * ip_hdr)
              */
             return check_ip_cidr(cidr, ip_hdr->ip_src.s_addr) ? DONT_SEND : SEND;
             break;
+
         case xXDest:
             return check_ip_cidr(cidr, ip_hdr->ip_dst.s_addr)  ? DONT_SEND : SEND;
+
         case xXBoth:
             return (check_ip_cidr(cidr, ip_hdr->ip_dst.s_addr)  &&
                     check_ip_cidr(cidr, ip_hdr->ip_src.s_addr) ) ? DONT_SEND : SEND;
             break;
+
         case xXEither:
             return (check_ip_cidr(cidr, ip_hdr->ip_dst.s_addr)  ||
                     check_ip_cidr(cidr, ip_hdr->ip_src.s_addr) ) ? DONT_SEND : SEND;
@@ -150,13 +158,16 @@ process_xX_by_cidr(int mode, tcpr_cidr_t * cidr, ipv4_hdr_t * ip_hdr)
         case xXSource:
             return check_ip_cidr(cidr, ip_hdr->ip_src.s_addr)  ? SEND : DONT_SEND;
             break;
+
         case xXDest:
             return check_ip_cidr(cidr, ip_hdr->ip_dst.s_addr)  ? SEND : DONT_SEND;
             break;
+
         case xXBoth:
             return (check_ip_cidr(cidr, ip_hdr->ip_dst.s_addr)  &&
                     check_ip_cidr(cidr, ip_hdr->ip_src.s_addr) ) ? SEND : DONT_SEND;
             break;
+
         case xXEither:
             return (check_ip_cidr(cidr, ip_hdr->ip_dst.s_addr)  ||
                     check_ip_cidr(cidr, ip_hdr->ip_src.s_addr) ) ? SEND : DONT_SEND;
@@ -175,11 +186,64 @@ process_xX_by_cidr(int mode, tcpr_cidr_t * cidr, ipv4_hdr_t * ip_hdr)
 
 }
 
-/*
- Local Variables:
- mode:c
- indent-tabs-mode:nil
- c-basic-offset:4
- End:
-*/
+int
+process_xX_by_cidr_ipv6(int mode, tcpr_cidr_t * cidr, ipv6_hdr_t * ip6_hdr)
+{
 
+    if (mode & xXExclude) {
+        /* Exclude mode */
+        switch (mode ^ xXExclude) {
+        case xXSource:
+            /* note: check_ip_cidr() returns TCPR_DIR_C2S for true, TCPR_DIR_S2C for false
+             * and NOT true/false or 1/0, etc!
+             */
+            return check_ip6_cidr(cidr, &ip6_hdr->ip_src) ? DONT_SEND : SEND;
+            break;
+
+        case xXDest:
+            return check_ip6_cidr(cidr, &ip6_hdr->ip_dst)  ? DONT_SEND : SEND;
+
+        case xXBoth:
+            return (check_ip6_cidr(cidr, &ip6_hdr->ip_dst)  &&
+                    check_ip6_cidr(cidr, &ip6_hdr->ip_src) ) ? DONT_SEND : SEND;
+            break;
+
+        case xXEither:
+            return (check_ip6_cidr(cidr, &ip6_hdr->ip_dst)  ||
+                    check_ip6_cidr(cidr, &ip6_hdr->ip_src) ) ? DONT_SEND : SEND;
+            break;
+        }
+    }
+    else {
+        /* Include Mode */
+        switch (mode) {
+        case xXSource:
+            return check_ip6_cidr(cidr, &ip6_hdr->ip_src)  ? SEND : DONT_SEND;
+            break;
+
+        case xXDest:
+            return check_ip6_cidr(cidr, &ip6_hdr->ip_dst)  ? SEND : DONT_SEND;
+            break;
+
+        case xXBoth:
+            return (check_ip6_cidr(cidr, &ip6_hdr->ip_dst)  &&
+                    check_ip6_cidr(cidr, &ip6_hdr->ip_src) ) ? SEND : DONT_SEND;
+            break;
+
+        case xXEither:
+            return (check_ip6_cidr(cidr, &ip6_hdr->ip_dst)  ||
+                    check_ip6_cidr(cidr, &ip6_hdr->ip_src) ) ? SEND : DONT_SEND;
+            break;
+        }
+    }
+
+    /* total failure */
+    if (mode &xXExclude) {
+        warn("Unable to determine action in CIDR filter mode.  Default: Don't Send.");
+        return DONT_SEND;
+    } else {
+        warn("Unable to determine action in CIDR filter mode.  Default: Send.");
+        return SEND;
+    }
+
+}
