@@ -210,6 +210,9 @@ enum tcpprep_mode {
 #endif
 #endif
 
+/* convert IPv6 Extention Header Len value to bytes */
+#define IPV6_EXTLEN_TO_BYTES(x) ((x * 4) + 8)
+
 #ifndef HAVE_UINT8_T
 typedef u_int8_t uint8_t
 typedef u_int16_t uint16_t
