@@ -1,4 +1,4 @@
-/* $Id: signal_handler.c 1898 2007-08-25 05:10:51Z aturner $ */
+/* $Id: signal_handler.c 2074 2008-11-09 08:42:24Z aturner $ */
 
 /*
  * Copyright (c) 2001-2007 Aaron Turner, Jeff Guttenfelder, Nathan Monteleone
@@ -88,7 +88,7 @@ suspend_handler(int signo)
     }
 
     if (gettimeofday(&suspend_start, NULL) < 0)
-        errx(1, "gettimeofday(): %s", strerror(errno));
+        errx(-1, "gettimeofday(): %s", strerror(errno));
 
     kill(getpid(), SIGSTOP);
 }
@@ -108,7 +108,7 @@ continue_handler(int signo)
     }
     
     if (gettimeofday(&suspend_end, NULL) < 0)
-        errx(1, "gettimeofday(): %s", strerror(errno));
+        errx(-1, "gettimeofday(): %s", strerror(errno));
 
     timersub(&suspend_end, &suspend_start, &suspend_delta);
     timeradd(&suspend_time, &suspend_delta, &suspend_time);
