@@ -1,4 +1,4 @@
-/* $Id: tcpbridge.c 2036 2008-05-20 16:38:20Z aturner $ */
+/* $Id: tcpbridge.c 2127 2009-01-10 06:06:46Z aturner $ */
 
 /*
  * Copyright (c) 2004-2005 Aaron Turner.
@@ -87,44 +87,42 @@ main(int argc, char *argv[])
 
    
     /* init tcpedit context */
-    if (tcpedit_init(&tcpedit, pcap_datalink(options.listen1)) < 0) {
-        errx(1, "Error initializing tcpedit: %s", tcpedit_geterr(tcpedit));
+    if (tcpedit_init(&tcpedit, pcap_datalink(options.pcap1)) < 0) {
+        errx(-1, "Error initializing tcpedit: %s", tcpedit_geterr(tcpedit));
     }
     
     /* parse the tcpedit args */
     rcode = tcpedit_post_args(&tcpedit);
     if (rcode < 0) {
-        errx(1, "Unable to parse args: %s", tcpedit_geterr(tcpedit));
+        errx(-1, "Unable to parse args: %s", tcpedit_geterr(tcpedit));
     } else if (rcode == 1) {
         warnx("%s", tcpedit_geterr(tcpedit));
     }
     
     if (tcpedit_validate(tcpedit) < 0) {
-        errx(1, "Unable to edit packets given options:\n%s",
+        errx(-1, "Unable to edit packets given options:\n%s",
                 tcpedit_geterr(tcpedit));
     }
 
 #ifdef ENABLE_VERBOSE
     if (options.verbose) {
         options.tcpdump = (tcpdump_t*)safe_malloc(sizeof(tcpdump_t));
-        tcpdump_open(options.tcpdump, options.listen1);
+        tcpdump_open(options.tcpdump, options.pcap1);
     }
 #endif
 
     if (gettimeofday(&begin, NULL) < 0)
-        err(1, "gettimeofday() failed");
+        err(-1, "gettimeofday() failed");
 
 
-    /* process packets from one or both interfaces */
-    do_bridge(tcpedit, options.listen1, options.listen2);
+    /* process packets */
+    do_bridge(&options, tcpedit);
 
     /* clean up after ourselves */
-    sendpacket_close(options.sp1);
-    pcap_close(options.listen1);
+    pcap_close(options.pcap1);
 
     if (options.unidir) {
-        sendpacket_close(options.sp2);
-        pcap_close(options.listen2);
+        pcap_close(options.pcap2);
     }
 
 #ifdef ENABLE_VERBOSE
@@ -157,7 +155,9 @@ void
 post_args(_U_ int argc, _U_ char *argv[])
 {
     char ebuf[SENDPACKET_ERRBUF_SIZE];
+    struct tcpr_ether_addr *eth_buff;
     char *intname;
+    sendpacket_t *sp;
 #ifdef ENABLE_PCAP_FINDALLDEVS
     interface_list_t *intlist = get_interface_list();
 #else
@@ -190,47 +190,77 @@ post_args(_U_ int argc, _U_ char *argv[])
 
 
     if ((intname = get_interface(intlist, OPT_ARG(INTF1))) == NULL)
-        errx(1, "Invalid interface name/alias: %s", OPT_ARG(INTF1));
+        errx(-1, "Invalid interface name/alias: %s", OPT_ARG(INTF1));
     
     options.intf1 = safe_strdup(intname);
 
     if (HAVE_OPT(INTF2)) {
         if ((intname = get_interface(intlist, OPT_ARG(INTF2))) == NULL)
-            errx(1, "Invalid interface name/alias: %s", OPT_ARG(INTF2));
+            errx(-1, "Invalid interface name/alias: %s", OPT_ARG(INTF2));
     
         options.intf2 = safe_strdup(intname);
     }
     
 
-    /* open up interfaces */
-    if ((options.sp1 = sendpacket_open(options.intf1, ebuf, TCPR_DIR_C2S)) == NULL)
-        errx(1, "Unable to open interface %s for sending: %s", options.intf1, ebuf);
+    if (HAVE_OPT(MAC)) {
+        int ct = STACKCT_OPT(MAC);
+        char **list = STACKLST_OPT(MAC);
+        int first = 1;
+        do {
+            char *p = *list++;
+            if (first)
+                mac2hex(p, (u_char *)options.intf1_mac, ETHER_ADDR_LEN);
+            else
+                mac2hex(p, (u_char *)options.intf2_mac, ETHER_ADDR_LEN);
+            first = 0;
+        } while (--ct > 0);
+    }
 
-    if ((options.listen1 = pcap_open_live(options.intf1, options.snaplen, 
+    /* 
+     * Figure out MAC addresses of sending interface(s)
+     * if user doesn't specify MAC address on CLI, query for it 
+     */
+    if (memcmp(options.intf1_mac, "\00\00\00\00\00\00", ETHER_ADDR_LEN) == 0) {
+        if ((sp = sendpacket_open(options.intf1, ebuf, TCPR_DIR_C2S)) == NULL)
+            errx(-1, "Unable to open interface %s: %s", options.intf1, ebuf);
+
+        if ((eth_buff = sendpacket_get_hwaddr(sp)) == NULL) {
+            warnx("Unable to get MAC address: %s", sendpacket_geterr(sp));
+            err(-1, "Please consult the man page for using the -M option.");
+        }
+        sendpacket_close(sp);
+        memcpy(options.intf1_mac, eth_buff, ETHER_ADDR_LEN);
+    }
+
+    if (memcmp(options.intf2_mac, "\00\00\00\00\00\00", ETHER_ADDR_LEN) == 0) {
+        if ((sp = sendpacket_open(options.intf2, ebuf, TCPR_DIR_S2C)) == NULL)
+            errx(-1, "Unable to open interface %s: %s", options.intf2, ebuf);
+
+        if ((eth_buff = sendpacket_get_hwaddr(sp)) == NULL) {
+            warnx("Unable to get MAC address: %s", sendpacket_geterr(sp));
+            err(-1, "Please consult the man page for using the -M option.");
+        }
+        sendpacket_close(sp);
+        memcpy(options.intf2_mac, eth_buff, ETHER_ADDR_LEN);        
+    }
+
+    /* 
+     * Open interfaces for sending & receiving 
+     */
+    if ((options.pcap1 = pcap_open_live(options.intf1, options.snaplen, 
                                           options.promisc, options.to_ms, ebuf)) == NULL)
-        errx(1, "Unable to open interface %s for recieving: %s", options.intf1, ebuf);
+        errx(-1, "Unable to open interface %s: %s", options.intf1, ebuf);
 
 
     if (strcmp(options.intf1, options.intf2) == 0)
-        errx(1, "Whoa tiger!  You don't want to use %s twice!", options.intf1);
-
-    if ((options.sp2 = sendpacket_open(options.intf2, ebuf, TCPR_DIR_S2C)) == NULL)
-        errx(1, "Unable to open interface %s for sending: %s", options.intf2, ebuf);
-
-    /* open 2nd interface for listening? (not in unidir mode) */
-    if (!options.unidir) {
-        if ((options.listen2 = pcap_open_live(options.intf2, options.snaplen,
-                                              options.promisc, options.to_ms, ebuf)) == NULL)
-            errx(1, "Unable to open interface %s for recieving: %s", options.intf2, ebuf);
-    }
-}
+        errx(-1, "Whoa tiger!  You don't want to use %s twice!", options.intf1);
 
 
-/*
- Local Variables:
- mode:c
- indent-tabs-mode:nil
- c-basic-offset:4
- End:
-*/
-
+    /* we always have to open the other pcap handle to send, but we may not listen */
+    if ((options.pcap2 = pcap_open_live(options.intf2, options.snaplen,
+                                          options.promisc, options.to_ms, ebuf)) == NULL)
+        errx(-1, "Unable to open interface %s: %s", options.intf2, ebuf);
+    
+    /* poll should be -1 to wait indefinitely */
+    options.poll_timeout = -1;
+}
\ No newline at end of file
