@@ -1,4 +1,4 @@
-/* $Id: parse_args.c 2211 2009-02-18 02:48:04Z aturner $ */
+/* $Id: parse_args.c 2314 2009-05-07 05:42:47Z aturner $ */
 
 /*
  * Copyright (c) 2006 Aaron Turner.
@@ -144,6 +144,10 @@ tcpedit_post_args(tcpedit_t **tcpedit_ex) {
     if (HAVE_OPT(TOS))
         tcpedit->tos = OPT_VALUE_TOS;
 
+    /* --flowlabel */
+    if (HAVE_OPT(FLOWLABEL))
+        tcpedit->flowlabel = OPT_VALUE_FLOWLABEL;
+
     /* --mtu */
     if (HAVE_OPT(MTU))
         tcpedit->mtu = OPT_VALUE_MTU;
