@@ -1,4 +1,4 @@
-/* $Id: en10mb.h 1841 2007-04-26 03:43:14Z aturner $ */
+/* $Id: en10mb.h 2105 2009-01-07 05:48:32Z aturner $ */
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
@@ -41,7 +41,7 @@ int dlt_en10mb_init(tcpeditdlt_t *ctx);
 int dlt_en10mb_cleanup(tcpeditdlt_t *ctx);
 int dlt_en10mb_parse_opts(tcpeditdlt_t *ctx);
 int dlt_en10mb_decode(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen);
-int dlt_en10mb_encode(tcpeditdlt_t *ctx, u_char **packet, int pktlen, tcpr_dir_t dir);
+int dlt_en10mb_encode(tcpeditdlt_t *ctx, u_char *packet, int pktlen, tcpr_dir_t dir);
 int dlt_en10mb_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen);
 u_char *dlt_en10mb_get_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen);
 u_char *dlt_en10mb_merge_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen, u_char *l3data);
