@@ -1,4 +1,4 @@
-/* $Id: raw.h 1963 2008-01-23 18:21:40Z aturner $ */
+/* $Id: raw.h 2105 2009-01-07 05:48:32Z aturner $ */
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
@@ -41,7 +41,7 @@ int dlt_raw_init(tcpeditdlt_t *ctx);
 int dlt_raw_cleanup(tcpeditdlt_t *ctx);
 int dlt_raw_parse_opts(tcpeditdlt_t *ctx);
 int dlt_raw_decode(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen);
-int dlt_raw_encode(tcpeditdlt_t *ctx, u_char **packet, int pktlen, tcpr_dir_t dir);
+int dlt_raw_encode(tcpeditdlt_t *ctx, u_char *packet, int pktlen, tcpr_dir_t dir);
 int dlt_raw_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen);
 u_char *dlt_raw_get_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen);
 u_char *dlt_raw_merge_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen, u_char *l3data);
