@@ -1,4 +1,4 @@
-/* $Id: dlt_utils.c 1832 2007-04-22 00:28:37Z aturner $ */
+/* $Id: dlt_utils.c 2076 2008-11-09 22:03:47Z aturner $ */
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
@@ -213,7 +213,9 @@ tcpedit_dlt_l3data_copy(tcpeditdlt_t *ctx, u_char *packet, int pktlen, int l2len
     assert(ctx);
     assert(packet);
     assert(pktlen);
-    assert(l2len < pktlen);
+
+    if (pktlen <= l2len)
+        return NULL;
     
 #ifdef FORCE_ALIGN
     /* 
