@@ -1,4 +1,4 @@
-/* $Id: tcpedit.c 2211 2009-02-18 02:48:04Z aturner $ */
+/* $Id: tcpedit.c 2317 2009-05-07 17:57:11Z aturner $ */
 
 /*
  * Copyright (c) 2001-2007 Aaron Turner.
@@ -71,8 +71,10 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         u_char **pktdata, tcpr_dir_t direction)
 {
     ipv4_hdr_t *ip_hdr = NULL;
+    ipv6_hdr_t *ip6_hdr = NULL;
     arp_hdr_t *arp_hdr = NULL;
     int l2len = 0, l2proto, retval = 0, dst_dlt, src_dlt, pktlen, lendiff;
+    int ipflags = 0, tclass = 0;
     int needtorecalc = 0;           /* did the packet change? if so, checksum */
     u_char *packet = *pktdata;
     assert(tcpedit);
@@ -134,8 +136,14 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
             return TCPEDIT_ERROR;
         }        
         dbgx(3, "Packet has an IPv4 header: %p...", ip_hdr);
+    } else if (l2proto == htons(ETHERTYPE_IP6)) {
+        ip6_hdr = (ipv6_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, src_dlt, packet, (*pkthdr)->caplen);
+        if (ip6_hdr == NULL) {
+            return TCPEDIT_ERROR;
+        }
+        dbgx(3, "Packet has an IPv6 header: %p...", ip6_hdr);
     } else {
-        dbgx(3, "Packet isn't IPv4: 0x%04x", l2proto);
+        dbgx(3, "Packet isn't IPv4 or IPv6: 0x%04x", l2proto);
         /* non-IP packets have a NULL ip_hdr struct */
         ip_hdr = NULL;
     }
@@ -152,26 +160,68 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
 
         /* rewrite TCP/UDP ports */
         if (tcpedit->portmap != NULL) {
-            if ((retval = rewrite_ports(tcpedit, &ip_hdr)) < 0)
+            if ((retval = rewrite_ipv4_ports(tcpedit, &ip_hdr)) < 0)
                 return TCPEDIT_ERROR;
             needtorecalc += retval;
         }
-
     }
+    /* IPv6 edits */
+    else if (ip6_hdr != NULL) {
+        /* rewrite the hop limit */
+        needtorecalc += rewrite_ipv6_hlim(tcpedit, ip6_hdr);
+
+        /* set traffic class? */
+        if (tcpedit->tclass > -1) {
+            /* calculate the bits */
+            tclass = tcpedit->tclass << 20;
+            
+            /* convert our 4 bytes to an int */
+            memcpy(&ipflags, &ip6_hdr->ip_flags, 4);
+            
+            /* strip out the old tclass bits */
+            ipflags = ntohl(ipflags) & 0xf00fffff;
+
+            /* add the tclass bits back */
+            ipflags += tclass; 
+            ipflags = htonl(ipflags);
+            memcpy(&ip6_hdr->ip_flags, &ipflags, 4);
+        }
+
+        /* set the flow label? */
+        if (tcpedit->flowlabel > -1) {
+            memcpy(&ipflags, &ip6_hdr->ip_flags, 4);
+            ipflags = ntohl(ipflags) & 0xfff00000;
+            ipflags += tcpedit->flowlabel;
+            ipflags = htonl(ipflags);
+            memcpy(&ip6_hdr->ip_flags, &ipflags, 4);
+        }
+
+        /* rewrite TCP/UDP ports */
+        if (tcpedit->portmap != NULL) {
+            if ((retval = rewrite_ipv6_ports(tcpedit, &ip6_hdr)) < 0)
+                return TCPEDIT_ERROR;
+            needtorecalc += retval;
+        }
+    }
+
     /* (Un)truncate or MTU truncate packet? */
     if (tcpedit->fixlen || tcpedit->mtu_truncate) {
-        if ((retval = untrunc_packet(tcpedit, *pkthdr, packet, ip_hdr)) < 0)
+        if ((retval = untrunc_packet(tcpedit, *pkthdr, packet, ip_hdr, ip6_hdr)) < 0)
             return TCPEDIT_ERROR;
         needtorecalc += retval;
     }
     
-    /* rewrite IP addresses in IPv4 or ARP */
+    /* rewrite IP addresses in IPv4/IPv6 or ARP */
     if (tcpedit->rewrite_ip) {
         /* IP packets */
         if (ip_hdr != NULL) {
             if ((retval = rewrite_ipv4l3(tcpedit, ip_hdr, direction)) < 0)
                 return TCPEDIT_ERROR;
             needtorecalc += retval;
+        } else if (ip6_hdr != NULL) {
+            if ((retval = rewrite_ipv6l3(tcpedit, ip6_hdr, direction)) < 0)
+                return TCPEDIT_ERROR;
+            needtorecalc += retval;
         }
 
         /* ARP packets */
@@ -196,6 +246,12 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
                 return TCPEDIT_ERROR;
             needtorecalc += retval;
 
+        } else if (ip6_hdr != NULL) {
+            if ((retval = randomize_ipv6(tcpedit, *pkthdr, packet,
+                    ip6_hdr)) < 0)
+                return TCPEDIT_ERROR;
+            needtorecalc += retval;
+
         /* ARP packets */
         } else if (l2proto == htons(ETHERTYPE_ARP)) {
             if (direction == TCPR_DIR_C2S) {
@@ -211,8 +267,14 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     }
 
     /* do we need to fix checksums? -- must always do this last! */
-    if ((tcpedit->fixcsum || needtorecalc) && (ip_hdr != NULL)) {
-        retval = fix_checksums(tcpedit, *pkthdr, ip_hdr);
+    if ((tcpedit->fixcsum || needtorecalc)) {
+        if (ip_hdr != NULL) {
+            retval = fix_ipv4_checksums(tcpedit, *pkthdr, ip_hdr);
+        } else if (ip6_hdr != NULL) {
+            retval = fix_ipv6_checksums(tcpedit, *pkthdr, ip6_hdr);
+        } else {
+            retval = TCPEDIT_OK;
+        }
         if (retval < 0) {
             return TCPEDIT_ERROR;
         } else if (retval == TCPEDIT_WARN) {
@@ -220,7 +282,6 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         }
     }
 
-    
     tcpedit_dlt_merge_l3data(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen, (u_char *)ip_hdr);
 
     tcpedit->runtime.total_bytes += (*pkthdr)->caplen;
@@ -243,7 +304,11 @@ tcpedit_init(tcpedit_t **tcpedit_ex, int dlt)
         return TCPEDIT_ERROR;
 
     tcpedit->mtu = DEFAULT_MTU; /* assume 802.3 Ethernet */
-    tcpedit->tos = -1; /* disabled by default */
+
+    /* disabled by default */
+    tcpedit->tos = -1;
+    tcpedit->tclass = -1;
+    tcpedit->flowlabel = -1;
  
     memset(&(tcpedit->runtime), 0, sizeof(tcpedit_runtime_t));
     tcpedit->runtime.dlt1 = dlt;
