@@ -1,4 +1,4 @@
-/* $Id: tcpr.h 2105 2009-01-07 05:48:32Z aturner $ */
+/* $Id: tcpr.h 2283 2009-05-03 06:47:44Z aturner $ */
 
 /*
  * Copyright (c) 2006-2007 Aaron Turner.
@@ -753,6 +753,9 @@ struct tcpr_in6_addr
     } __u6_addr;            /* 128-bit IP6 address */
 };
 #define tcpr_s6_addr __u6_addr.__u6_addr8
+#define tcpr_s6_addr8 __u6_addr.__u6_addr8
+#define tcpr_s6_addr16 __u6_addr.__u6_addr16
+#define tcpr_s6_addr32 __u6_addr.__u6_addr32
 
 /*
  *  IPv6 header
@@ -766,9 +769,20 @@ struct tcpr_ipv6_hdr
     u_int8_t ip_nh;           /* next header */
     u_int8_t ip_hl;           /* hop limit */
     struct tcpr_in6_addr ip_src, ip_dst; /* source and dest address */
-
 };
 
+struct tcpr_ipv6_ext_hdr_base
+{
+    u_int8_t ip_nh;          /* next header */
+    u_int8_t ip_len;         /* length of header in 8 octet units (sans 1st) */
+    /* some more bytes are always here, but we don't know what kind */
+};
+
+#define TCPR_IPV6_NH_NO_NEXT 59
+#define TCPR_IPV6_NH_IPV6    41
+#define TCPR_IPV6_NH_ESP     50
+#define TCPR_IPV6_NH_AH      51
+
 /*
  *  IPv6 frag header
  *  Internet Protocol, version 6
